@@ -26,6 +26,12 @@ from . import _pykeywords
 
 from ._jvmfinder import *
 
+# This import is required to bootstrap importlib, _jpype uses importlib.util
+# but on some systems it may not load properly from C.  To make sure it gets
+# loaded properly we are going to force the issue even through we don't
+# use it in this module.  (Do not remove)
+from importlib import util as _util
+
 __all__ = [
     'isJVMStarted', 'startJVM', 'shutdownJVM',
     'getDefaultJVMPath', 'getJVMVersion', 'isThreadAttachedToJVM', 'attachThreadToJVM',
@@ -116,6 +122,10 @@ def _handleClassPath(clsList):
 _JVM_started = False
 
 
+def interactive():
+    return bool(getattr(sys, 'ps1', sys.flags.interactive))
+
+
 def startJVM(*args, **kwargs):
     """
     Starts a Java Virtual Machine.  Without options it will start
@@ -143,12 +153,11 @@ def startJVM(*args, **kwargs):
         will globally change the behavior of all calls using
         strings, and a value of True is NOT recommended for newly
         developed code.
-
-        The default value for this option during 0.7 series is
-        True.  The option will be False starting in 0.8. A
-        warning will be issued if this option is not specified
-        during the transition period.
-
+      interrupt (bool): Option to install ^C signal handlers.
+        If True then ^C will stop the process, else ^C will
+        transfer control to Python rather than halting.  If
+        not specified will be False if Python is started as
+        an interactive shell.
 
     Raises:
       OSError: if the JVM cannot be started or is already running.
@@ -202,6 +211,7 @@ def startJVM(*args, **kwargs):
 
     ignoreUnrecognized = kwargs.pop('ignoreUnrecognized', False)
     convertStrings = kwargs.pop('convertStrings', False)
+    interrupt = kwargs.pop('interrupt', not interactive())
 
     if kwargs:
         raise TypeError("startJVM() got an unexpected keyword argument '%s'"
@@ -209,8 +219,8 @@ def startJVM(*args, **kwargs):
 
     try:
         _jpype.startup(jvmpath, tuple(args),
-                       ignoreUnrecognized, convertStrings)
-        _JVM_started = True
+                       ignoreUnrecognized, convertStrings, interrupt)
+        initializeResources()
     except RuntimeError as ex:
         source = str(ex)
         if "UnsupportedClassVersion" in source:
@@ -222,6 +232,9 @@ def startJVM(*args, **kwargs):
                     jvmpath, version)) from ex
         raise
 
+
+def initializeResources():
+    global _JVM_started
     _jpype._java_lang_Class = None
     _jpype._java_lang_Object = _jpype.JClass("java.lang.Object")
     _jpype._java_lang_Throwable = _jpype.JClass("java.lang.Throwable")
@@ -299,6 +312,12 @@ def startJVM(*args, **kwargs):
     _jpype.JClass('org.jpype.JPypeKeywords').setKeywords(
         list(_pykeywords._KEYWORDS))
 
+    _jpype.JPypeContext = _jpype.JClass('org.jpype.JPypeContext').getInstance()
+    _jpype.JPypeClassLoader = _jpype.JPypeContext.getClassLoader()
+
+    # Everything successed so started is now true.
+    _JVM_started = True
+
 
 def shutdownJVM():
     """ Shuts down the JVM.
