@@ -1,5 +1,4 @@
-#*****************************************************************************
-#   Copyright 2004-2008 Steve Menard
+# *****************************************************************************
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -13,27 +12,21 @@
 #   See the License for the specific language governing permissions and
 #   limitations under the License.
 #
-#*****************************************************************************
-import sys
-
+#   See NOTICE file for details.
+#
+# *****************************************************************************
 import _jpype
 
+__all__ = ['convertToDirectBuffer']
 
-if sys.version_info < (2, 7):
-    _mem_view = _jpype.memoryview  # own memoryview implementation
-else:
-    _mem_view = memoryview
-
-
-def _initialize() :
-    pass
 
 def convertToDirectBuffer(obj):
     __doc__ = '''Efficiently convert all array.array and numpy ndarray types, string and unicode to java.nio.Buffer objects.'''
 
-    memoryview_of_obj = _mem_view(obj)
+    memoryview_of_obj = memoryview(obj)
 
     if memoryview_of_obj.readonly:
-        raise ValueError("Memoryview must be writable for wrapping in a byte buffer")
+        raise ValueError(
+            "Memoryview must be writable for wrapping in a byte buffer")
 
     return _jpype.convertToDirectBuffer(memoryview_of_obj)
