@@ -56,17 +56,28 @@ jvalue JPMatch::convert()
 	return conversion->convert(*this);
 }
 
-JPMethodMatch::JPMethodMatch(JPJavaFrame &frame, JPPyObjectVector& args)
-: argument(args.size())
+JPMethodMatch::JPMethodMatch(JPJavaFrame &frame, JPPyObjectVector& args, bool callInstance)
+: m_Arguments(args.size())
 {
-	type = JPMatch::_none;
-	isVarIndirect = false;
-	overload = 0;
-	offset = 0;
-	skip = 0;
+	m_Type = JPMatch::_none;
+	m_IsVarIndirect = false;
+	m_Overload = 0;
+	m_Offset = 0;
+	m_Skip = 0;
+	m_Hash = callInstance ? 0 : 1000;
 	for (size_t i = 0; i < args.size(); ++i)
 	{
-		argument[i] = JPMatch(&frame, args[i]);
+		PyObject *arg = args[i];
+		m_Arguments[i] = JPMatch(&frame, arg);
+
+		// This is an LCG used to compute a hash code for the incoming
+		// arguments using (A*X+A_i) mod2^64 where A_i is the address of each
+		// type the argument list.  The hash will be checked to avoid needing
+		// to resolve the method if the same overload is called twice. There
+		// is only a speed cost if there is a collision, so we don't need to
+		// prove this is a perfect hash function.
+		m_Hash *= 0x10523C01;
+		m_Hash += (long) (Py_TYPE(arg));
 	}
 }
 
@@ -195,7 +206,7 @@ public:
 		JP_TRACE_OUT;
 	}
 
-	virtual void getInfo(JPClass *cls, JPConversionInfo &info)
+	virtual void getInfo(JPClass *cls, JPConversionInfo &info) override
 	{
 		PyList_Append(info.attributes, JPPyString::fromStringUTF8(attribute_).get());
 	}
@@ -321,7 +332,7 @@ class JPHintsConversion : public JPConversion
 {
 public:
 
-	virtual JPMatch::Type matches(JPClass *cls, JPMatch &match)
+	virtual JPMatch::Type matches(JPClass *cls, JPMatch &match) override
 	{
 		PyJPClassHints *pyhints = (PyJPClassHints*) cls->getHints();
 		// GCOVR_EXCL_START
@@ -348,7 +359,7 @@ public:
 		hints->getInfo(cls, info);
 	}
 
-	virtual jvalue convert(JPMatch &match)
+	virtual jvalue convert(JPMatch &match) override
 	{
 		return jvalue();
 	}
@@ -436,6 +447,62 @@ public:
 	}
 } _byteArrayConversion;
 
+class JPConversionBuffer : public JPConversion
+{
+public:
+
+	virtual JPMatch::Type matches(JPClass *cls, JPMatch &match) override
+	{
+		JP_TRACE_IN("JPConversionBuffer::matches");
+		JPArrayClass *acls = (JPArrayClass*) cls;
+		JPClass *componentType = acls->getComponentType();
+		if ( !componentType->isPrimitive())
+			return match.type = JPMatch::_none;
+		// If is isn't a buffer we can skip
+		JPPyBuffer	buffer(match.object, PyBUF_ND | PyBUF_FORMAT);
+		if (!buffer.valid())
+		{
+			PyErr_Clear();
+			return match.type = JPMatch::_none;
+		}
+
+		// If it is a buffer we only need to test the first item in the list
+		JPPySequence seq = JPPySequence::use(match.object);
+		jlong length = seq.size();
+		match.type = JPMatch::_implicit;
+		if (length > 0)
+		{
+			JPPyObject item = seq[0];
+			JPMatch imatch(match.frame, item.get());
+			componentType->findJavaConversion(imatch);
+			if (imatch.type < match.type)
+				match.type = imatch.type;
+		}
+		match.closure = cls;
+		match.conversion = bufferConversion;
+		return match.type;
+		JP_TRACE_OUT;
+	}
+
+	virtual void getInfo(JPClass *cls, JPConversionInfo &info) override
+	{
+		// This will be covered by Sequence
+	}
+
+	virtual jvalue convert(JPMatch &match) override
+	{
+		JPJavaFrame frame(*match.frame);
+		jvalue res;
+		JPArrayClass *acls = (JPArrayClass *) match.closure;
+		jsize length = (jsize) PySequence_Length(match.object);
+		JPClass *ccls = acls->getComponentType();
+		jarray array = ccls->newArrayOf(frame, (jsize) length);
+		ccls->setArrayRange(frame, array, 0, length, 1, match.object);
+		res.l = frame.keep(array);
+		return res;
+	}
+}  _bufferConversion;
+
 class JPConversionSequence : public JPConversion
 {
 public:
@@ -483,16 +550,10 @@ public:
 		JPJavaFrame frame(*match.frame);
 		jvalue res;
 		JPArrayClass *acls = (JPArrayClass *) match.closure;
-		JP_TRACE("sequence");
-		JPPySequence seq = JPPySequence::use(match.object);
-		jsize length = (jsize) seq.size();
-
-		jarray array = acls->getComponentType()->newArrayInstance(frame, (jsize) length);
-		for (jsize i = 0; i < length; i++)
-		{
-			JPPyObject item = seq[i];
-			acls->getComponentType()->setArrayItem(frame, array, i, item.get());
-		}
+		jsize length = (jsize) PySequence_Length(match.object);
+		JPClass *ccls = acls->getComponentType();
+		jarray array = ccls->newArrayOf(frame, (jsize) length);
+		ccls->setArrayRange(frame, array, 0, length, 1, match.object);
 		res.l = frame.keep(array);
 		return res;
 	}
@@ -720,7 +781,7 @@ public:
 		PyList_Append(info.implicit, proto.get());
 	}
 
-	jvalue convert(JPMatch &match)  override
+	jvalue convert(JPMatch &match) override
 	{
 		PyTypeObject* type = Py_TYPE(match.object);
 		JPJavaFrame *frame = match.frame;
@@ -934,6 +995,7 @@ public:
 JPConversion *hintsConversion = &_hintsConversion;
 JPConversion *charArrayConversion = &_charArrayConversion;
 JPConversion *byteArrayConversion = &_byteArrayConversion;
+JPConversion *bufferConversion = &_bufferConversion;
 JPConversion *sequenceConversion = &_sequenceConversion;
 JPConversion *nullConversion = &_nullConversion;
 JPConversion *classConversion = &_classConversion;
