@@ -10,22 +10,27 @@
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
-  
+
   See NOTICE file for details.
 **************************************************************************** */
 package org.jpype;
 
+import java.io.File;
 import java.lang.reflect.Array;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
 import java.nio.Buffer;
 import java.nio.ByteOrder;
+import java.nio.file.Files;
+import java.nio.file.Path;
+import java.nio.file.Paths;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
 import java.util.Map;
 import java.util.concurrent.atomic.AtomicInteger;
+import org.jpype.classloader.DynamicClassLoader;
 import org.jpype.manager.TypeFactory;
 import org.jpype.manager.TypeFactoryNative;
 import org.jpype.manager.TypeManager;
@@ -68,23 +73,21 @@ import org.jpype.ref.JPypeReferenceQueue;
 public class JPypeContext
 {
 
-  public final String VERSION = "1.0.1";
+  public final String VERSION = "1.2.1";
 
-  private static JPypeContext instance = null;
+  private static JPypeContext INSTANCE = new JPypeContext();
   // This is the C++ portion of the context.
   private long context;
   private TypeFactory typeFactory;
   private TypeManager typeManager;
-  private JPypeReferenceQueue referenceQueue;
-  private ClassLoader bootLoader;
-  private AtomicInteger shutdownFlag = new AtomicInteger();
-  private AtomicInteger proxyCount = new AtomicInteger();
-  private List<Thread> shutdownHooks = new ArrayList<>();
-  private List<Runnable> postHooks = new ArrayList<>();
+  private DynamicClassLoader classLoader;
+  private final AtomicInteger shutdownFlag = new AtomicInteger();
+  private final List<Thread> shutdownHooks = new ArrayList<>();
+  private final List<Runnable> postHooks = new ArrayList<>();
 
   static public JPypeContext getInstance()
   {
-    return instance;
+    return INSTANCE;
   }
 
   /**
@@ -94,22 +97,33 @@ public class JPypeContext
    * @param bootLoader is the classloader holding JPype resources.
    * @return the created context.
    */
-  public static JPypeContext createContext(long context, ClassLoader bootLoader)
+  static JPypeContext createContext(long context, ClassLoader bootLoader, String nativeLib, boolean interrupt)
   {
-    instance = new JPypeContext();
+    if (nativeLib != null)
+    {
+      System.load(nativeLib);
+    }
+    INSTANCE.context = context;
+    INSTANCE.classLoader = (DynamicClassLoader) bootLoader;
+    INSTANCE.typeFactory = new TypeFactoryNative();
+    INSTANCE.typeManager = new TypeManager(context, INSTANCE.typeFactory);
+    INSTANCE.initialize(interrupt);
 
-    instance.context = context;
-    instance.bootLoader = bootLoader;
-    instance.typeFactory = new TypeFactoryNative();
-    instance.typeManager = new TypeManager(context, instance.typeFactory);
-    instance.typeManager.typeFactory = instance.typeFactory;
+    scanExistingJars();
+    return INSTANCE;
+  }
 
-    instance.referenceQueue = new JPypeReferenceQueue(context);
+  private JPypeContext()
+  {
+  }
 
+  void initialize(boolean interrupt)
+  {
     // Okay everything is setup so lets give it a go.
-    instance.typeManager.init();
-    instance.referenceQueue.start();
-    JPypeSignal.installHandlers();
+    this.typeManager.init();
+    JPypeReferenceQueue.getInstance().start();
+    if (!interrupt)
+      JPypeSignal.installHandlers();
 
     // Install a shutdown hook to clean up Python resources.
     Runtime.getRuntime().addShutdownHook(new Thread(new Runnable()
@@ -117,11 +131,10 @@ public class JPypeContext
       @Override
       public void run()
       {
-        instance.shutdown();
+        INSTANCE.shutdown();
       }
     }));
 
-    return instance;
   }
 
   /**
@@ -192,7 +205,6 @@ public class JPypeContext
       {
         if (t1 == t || t.isDaemon())
           continue;
-//      if (t.getState() == Thread.State.RUNNABLE)
         t.interrupt();
       }
 
@@ -202,27 +214,29 @@ public class JPypeContext
 
       // Wait for any unregistered proxies to finish so that we don't yank
       // the rug out from under them result in a segfault.
-      while (this.proxyCount.get() > 0)
-      {
-        try
-        {
-          Thread.sleep(10);
-        } catch (InterruptedException ex)
-        {
-        }
-      }
-
-//    // Check to see if who is alive
-//    threads = Thread.getAllStackTraces();
-//    System.out.println("Check for remaining");
-//    for (Thread t : threads.keySet())
-//    {
-//      System.out.println("  " + t.getName() + " " + t.getState());
-//      for (StackTraceElement e : t.getValue())
+//      while (this.proxyCount.get() > 0)
 //      {
-//        System.out.println("    " + e.getClassName());
+//        try
+//        {
+//          Thread.sleep(10);
+//        } catch (InterruptedException ex)
+//        {
+//        }
+//      }
+//      // Check to see if who is alive
+//      threads = Thread.getAllStackTraces();
+//      System.out.println("Check for remaining");
+//      for (Thread t : threads.keySet())
+//      {
+//        // Daemon threads don't count for shutdown so skip them.
+//        if (t.isDaemon())
+//          continue;
+//        System.out.println("  " + t.getName() + " " + t.getState() + " " + t.isDaemon());
+//        for (StackTraceElement e : t.getStackTrace())
+//        {
+//          System.out.println("    " + e.getClassName());
+//        }
 //      }
-//    }
     } catch (Throwable th)
     {
     }
@@ -230,7 +244,7 @@ public class JPypeContext
     // Release all Python references
     try
     {
-      this.referenceQueue.stop();
+      JPypeReferenceQueue.getInstance().stop();
     } catch (Throwable th)
     {
     }
@@ -248,6 +262,7 @@ public class JPypeContext
     {
       run.run();
     }
+
   }
 
   static native void onShutdown(long ctxt);
@@ -277,9 +292,9 @@ public class JPypeContext
     return context;
   }
 
-  public ClassLoader getBootLoader()
+  public ClassLoader getClassLoader()
   {
-    return this.bootLoader;
+    return this.classLoader;
   }
 
   public TypeFactory getTypeFactory()
@@ -292,11 +307,6 @@ public class JPypeContext
     return this.typeManager;
   }
 
-  public JPypeReferenceQueue getReferenceQueue()
-  {
-    return this.referenceQueue;
-  }
-
   /**
    * Add a hook to run after Python interface is shutdown.
    *
@@ -447,25 +457,50 @@ public class JPypeContext
     return shutdownFlag.get() > 0;
   }
 
-  public void incrementProxy()
+//  public void incrementProxy()
+//  {
+//    proxyCount.incrementAndGet();
+//  }
+//
+//  public void decrementProxy()
+//  {
+//    proxyCount.decrementAndGet();
+//  }
+  /**
+   * Clear the current interrupt.
+   *
+   * @param x is true if an exception should be thrown.
+   * @throws InterruptedException
+   */
+  public static void clearInterrupt(boolean x) throws InterruptedException
   {
-    proxyCount.incrementAndGet();
-  }
-
-  public void decrementProxy()
-  {
-    proxyCount.decrementAndGet();
-  }
-
-  public static class PyExceptionProxy extends RuntimeException
-  {
-
-    long cls, value;
-
-    public PyExceptionProxy(long l0, long l1)
+    try
     {
-      cls = l0;
-      value = l1;
+      Thread th = Thread.currentThread();
+
+      // Only relevant if this is the main thread for signal handling
+      if (th != JPypeSignal.main)
+        return;
+
+      // Unconditionally clear the interrupt flag if we are called from 
+      // C++.  This happens when a field get() or method call() is 
+      // invoked.
+      if (!x)
+        JPypeSignal.acknowledgePy();
+
+      // Check if this thread is interrupted
+      if (th.isInterrupted())
+      {
+        // Clear the flag in C++
+        JPypeSignal.acknowledgePy();
+
+        // Clear the flag in Java
+        Thread.sleep(1);
+      }
+    } catch (InterruptedException ex)
+    {
+      if (x)
+        throw ex;
     }
   }
 
@@ -518,7 +553,7 @@ public class JPypeContext
     s = JPypeKeywords.safepkg(s);
     if (!JPypePackageManager.isPackage(s))
       return null;
-    return new JPypePackage(s, JPypePackageManager.getContentMap(s));
+    return new JPypePackage(s);
   }
 
   /**
@@ -608,4 +643,14 @@ public class JPypeContext
     }
   }
 
+  private static void scanExistingJars()
+  {
+    // Scan existing jars for missing directory entries
+    String[] paths = System.getProperty("java.class.path").split(File.pathSeparator);
+    for (String path : paths)
+    {
+      INSTANCE.classLoader.scanJar(Paths.get(path));
+    }
+  }
+
 }
