@@ -16,10 +16,10 @@
 #include "jpype.h"
 #include "pyjp.h"
 #include "jp_arrayclass.h"
-#include "jp_reference_queue.h"
 #include "jp_primitive_accessor.h"
 #include "jp_gc.h"
 #include "jp_stringtype.h"
+#include "jp_classloader.h"
 
 void PyJPModule_installGC(PyObject* module);
 
@@ -73,7 +73,7 @@ PyObject* _JMethodCode = NULL;
 PyObject* _JObjectKey = NULL;
 PyObject* _JVMNotRunning = NULL;
 
-static void PyJPModule_loadResources(PyObject* module)
+void PyJPModule_loadResources(PyObject* module)
 {
 	// Note that if any resource is missing the user will get
 	// the message:
@@ -126,7 +126,7 @@ static void PyJPModule_loadResources(PyObject* module)
 	}	catch (JPypeException&)  // GCOVR_EXCL_LINE
 	{
 		// GCOVR_EXCL_START
-		Py_SetStringWithCause(PyExc_RuntimeError, "JPype resource is missing");
+		PyJP_SetStringWithCause(PyExc_RuntimeError, "JPype resource is missing");
 		JP_RAISE_PYTHON();
 		// GCOVR_EXCL_STOP
 	}
@@ -137,9 +137,11 @@ extern "C"
 {
 #endif
 
+
 // GCOVR_EXCL_START
 // This is used exclusively during startup
-void Py_SetStringWithCause(PyObject *exception,
+
+void PyJP_SetStringWithCause(PyObject *exception,
 		const char *str)
 {
 	// See _PyErr_TrySetFromCause
@@ -161,7 +163,7 @@ void Py_SetStringWithCause(PyObject *exception,
 }
 // GCOVR_EXCL_STOP
 
-PyObject* Py_GetAttrDescriptor(PyTypeObject *type, PyObject *attr_name)
+PyObject* PyJP_GetAttrDescriptor(PyTypeObject *type, PyObject *attr_name)
 {
 	JP_PY_TRY("Py_GetAttrDescriptor");
 	if (type->tp_mro == NULL)
@@ -194,7 +196,7 @@ PyObject* Py_GetAttrDescriptor(PyTypeObject *type, PyObject *attr_name)
 	JP_PY_CATCH(NULL); // GCOVR_EXCL_LINE
 }
 
-int Py_IsSubClassSingle(PyTypeObject* type, PyTypeObject* obj)
+int PyJP_IsSubClassSingle(PyTypeObject* type, PyTypeObject* obj)
 {
 	if (type == NULL || obj == NULL)
 		return 0;  // GCOVR_EXCL_LINE
@@ -206,13 +208,16 @@ int Py_IsSubClassSingle(PyTypeObject* type, PyTypeObject* obj)
 	return PyTuple_GetItem(mro1, n1 - n2) == (PyObject*) type;
 }
 
-int Py_IsInstanceSingle(PyObject* obj, PyTypeObject* type)
+int PyJP_IsInstanceSingle(PyObject* obj, PyTypeObject* type)
 {
 	if (type == NULL || obj == NULL)
 		return 0; // GCOVR_EXCL_LINE
-	return Py_IsSubClassSingle(type, Py_TYPE(obj));
+	return PyJP_IsSubClassSingle(type, Py_TYPE(obj));
 }
 
+#ifndef ANDROID
+extern JNIEnv *Android_JNI_GetEnv();
+
 static PyObject* PyJPModule_startup(PyObject* module, PyObject* pyargs)
 {
 	JP_PY_TRY("PyJPModule_startup");
@@ -221,9 +226,10 @@ static PyObject* PyJPModule_startup(PyObject* module, PyObject* pyargs)
 	PyObject* vmPath;
 	char ignoreUnrecognized = true;
 	char convertStrings = false;
+	char interrupt = false;
 
-	if (!PyArg_ParseTuple(pyargs, "OO!bb", &vmPath, &PyTuple_Type, &vmOpt,
-			&ignoreUnrecognized, &convertStrings))
+	if (!PyArg_ParseTuple(pyargs, "OO!bbb", &vmPath, &PyTuple_Type, &vmOpt,
+			&ignoreUnrecognized, &convertStrings, &interrupt))
 		return NULL;
 
 	if (!(JPPyString::check(vmPath)))
@@ -264,9 +270,8 @@ static PyObject* PyJPModule_startup(PyObject* module, PyObject* pyargs)
 
 	// install the gc hook
 	PyJPModule_installGC(module);
-
 	PyJPModule_loadResources(module);
-	JPContext_global->startJVM(cVmPath, args, ignoreUnrecognized != 0, convertStrings != 0);
+	JPContext_global->startJVM(cVmPath, args, ignoreUnrecognized != 0, convertStrings != 0, interrupt != 0);
 
 	Py_RETURN_NONE;
 	JP_PY_CATCH(NULL);
@@ -279,12 +284,15 @@ static PyObject* PyJPModule_shutdown(PyObject* obj)
 	Py_RETURN_NONE;
 	JP_PY_CATCH(NULL);
 }
+#endif
 
 static PyObject* PyJPModule_isStarted(PyObject* obj)
 {
 	return PyBool_FromLong(JPContext_global->isRunning());
 }
 
+#ifndef ANDROID
+
 static PyObject* PyJPModule_attachThread(PyObject* obj)
 {
 	JP_PY_TRY("PyJPModule_attachThread");
@@ -309,6 +317,7 @@ static PyObject* PyJPModule_detachThread(PyObject* obj)
 	Py_RETURN_NONE;
 	JP_PY_CATCH(NULL);
 }
+#endif
 
 static PyObject* PyJPModule_isThreadAttached(PyObject* obj)
 {
@@ -347,7 +356,7 @@ static PyObject* PyJPModule_convertToDirectByteBuffer(PyObject* self, PyObject*
 		v.l = frame.NewDirectByteBuffer(vw.view->buf, vw.view->len);
 
 		// Bind lifespan of the view to the java object.
-		context->getReferenceQueue()->registerRef(v.l, vw.view, &releaseView);
+		frame.registerRef(v.l, vw.view, &releaseView);
 		vw.view = 0;
 		JPClass *type = frame.findClassForObject(v.l);
 		return type->convertToPythonObject(frame, v, false).keep();
@@ -536,7 +545,7 @@ PyObject *PyJPModule_gcStats(PyObject* module, PyObject *obj)
 }
 // GCOVR_EXCL_STOP
 
-PyObject* PyJPModule_isPackage(PyObject *module, PyObject *pkg)
+static PyObject* PyJPModule_isPackage(PyObject *module, PyObject *pkg)
 {
 	JP_PY_TRY("PyJPModule_isPackage");
 	if (!PyUnicode_Check(pkg))
@@ -550,8 +559,10 @@ PyObject* PyJPModule_isPackage(PyObject *module, PyObject *pkg)
 	JP_PY_CATCH(NULL); // GCOVR_EXCL_LINE
 }
 
-// GCOVR_EXCL_START
 
+#if 0
+// GCOVR_EXCL_START
+// This code was used in testing the Java slot memory layout.  It serves no purpose outside of debugging that issue.
 PyObject* examine(PyObject *module, PyObject *other)
 {
 	JP_PY_TRY("examine");
@@ -587,12 +598,12 @@ PyObject* examine(PyObject *module, PyObject *other)
 	printf("    free: %p\n", type->tp_free);
 	printf("    finalize: %p\n", type->tp_finalize);
 	printf("======\n");
-	fflush(stdout);
 
 	return PyBool_FromLong(ret);
 	JP_PY_CATCH(NULL);
 }
 // GCOVR_EXCL_STOP
+#endif
 
 // GCOVR_EXCL_START
 int _PyJPModule_trace = 0;
@@ -624,24 +635,42 @@ static PyObject* PyJPModule_fault(PyObject *module, PyObject *args)
 }
 #endif
 
+#ifdef ANDROID
+
+static PyObject *PyJPModule_bootstrap(PyObject *module)
+{
+	// After all the internals are created we can connect the API with the internal module
+	JNIEnv * env = Android_JNI_GetEnv();
+	JPContext_global->attachJVM(env);
+	PyJPModule_installGC(module);
+	PyJPModule_loadResources(module);
+	Py_RETURN_NONE;
+}
+#endif
+
 static PyMethodDef moduleMethods[] = {
 	// Startup and initialization
 	{"isStarted", (PyCFunction) PyJPModule_isStarted, METH_NOARGS, ""},
+#ifdef ANDROID
+	{"bootstrap", (PyCFunction) PyJPModule_bootstrap, METH_NOARGS, ""},
+#else
 	{"startup", (PyCFunction) PyJPModule_startup, METH_VARARGS, ""},
 	//	{"attach", (PyCFunction) (&PyJPModule_attach), METH_VARARGS, ""},
 	{"shutdown", (PyCFunction) PyJPModule_shutdown, METH_NOARGS, ""},
+#endif
 	{"_getClass", (PyCFunction) PyJPModule_getClass, METH_O, ""},
 	{"_hasClass", (PyCFunction) PyJPModule_hasClass, METH_O, ""},
-	{"examine", (PyCFunction) examine, METH_O, ""},
 	{"_newArrayType", (PyCFunction) PyJPModule_newArrayType, METH_VARARGS, ""},
 	{"_collect", (PyCFunction) PyJPModule_collect, METH_VARARGS, ""},
 	{"gcStats", (PyCFunction) PyJPModule_gcStats, METH_NOARGS, ""},
 
 	// Threading
 	{"isThreadAttachedToJVM", (PyCFunction) PyJPModule_isThreadAttached, METH_NOARGS, ""},
+#ifndef ANDROID
 	{"attachThreadToJVM", (PyCFunction) PyJPModule_attachThread, METH_NOARGS, ""},
 	{"detachThreadFromJVM", (PyCFunction) PyJPModule_detachThread, METH_NOARGS, ""},
 	{"attachThreadAsDaemon", (PyCFunction) PyJPModule_attachThreadAsDaemon, METH_NOARGS, ""},
+#endif
 
 	//{"dumpJVMStats", (PyCFunction) (&PyJPModule_dumpJVMStats), METH_NOARGS, ""},
 
@@ -673,18 +702,23 @@ PyMODINIT_FUNC PyInit__jpype()
 {
 	JP_PY_TRY("PyInit__jpype");
 	JPContext_global = new JPContext();
+
+#if PY_VERSION_HEX<0x03070000
 	// This is required for python versions prior to 3.7.
 	// It is called by the python initialization starting from 3.7,
-	// but is safe to call afterwards.
+	// but is safe to call afterwards.  Starting 3.9 this issues a 
+	// deprecation warning.
 	PyEval_InitThreads();
+#endif
 
 	// Initialize the module (depends on python version)
 	PyObject* module = PyModule_Create(&moduledef);
 	// PyJPModule = module;
 	Py_INCREF(module);
 	PyJPModule = module;
-	PyModule_AddStringConstant(module, "__version__", "1.0.1");
+	PyModule_AddStringConstant(module, "__version__", "1.2.1");
 
+	PyJPClassMagic = PyDict_New();
 	// Initialize each of the python extension types
 	PyJPClass_initType(module);
 	PyJPObject_initType(module);
@@ -701,6 +735,7 @@ PyMODINIT_FUNC PyInit__jpype()
 	PyJPChar_initType(module);
 
 	_PyJPModule_trace = true;
+
 	return module;
 	JP_PY_CATCH(NULL); // GCOVR_EXCL_LINE
 }
@@ -811,7 +846,7 @@ static PyObject *PyJPModule_convertBuffer(JPPyBuffer& buffer, PyObject *dtype)
 	// Convert the shape
 	Py_ssize_t subs = 1;
 	Py_ssize_t base = 1;
-	jintArray jdims = (jintArray) context->_int->newArrayInstance(frame, view.ndim);
+	jintArray jdims = (jintArray) context->_int->newArrayOf(frame, view.ndim);
 	if (view.shape != NULL)
 	{
 		JPPrimitiveArrayAccessor<jintArray, jint*> accessor(frame, jdims,
